#!/usr/bin/env python2.7
# -*- coding: utf-8 -*-

__author__ = 'yarnaid'

import properties
from ext.angles import Zone, Angle
from pixelmap import gPixelMap as PixelMap
from pointsource import PointSource
from tools import convertion as convert
from tools import colorer
from tools.logger import logger
import tools.tools
import os
from cl import Cl


mappat_path = os.path.join(tools.tools.glesp_exec, tools.tools.glesp['mappat'])
if not os.path.exists(mappat_path):
    raise ImportError('Cannot import module {}. No mappat binary found in {}'.format(
        __name__, tools.tools.glesp_exec
    ))


__author__ = 'yarnaid'

import unittest
import test.test_pixelmap as test_pixelmap
import test.test_properties as test_properties
import tools.test.test_convertation as test_convertation
import tools.test.test_tools as test_tools


loader = unittest.TestLoader()

suite_glespy = loader.loadTestsFromModule(test_pixelmap)
suite_glespy.addTest(loader.loadTestsFromModule(test_properties))

suite_tools = loader.loadTestsFromModule(test_convertation)
suite_tools = loader.loadTestsFromModule(test_tools)

runner = unittest.TextTestRunner()
# runner.run(suite_tools)
# runner.run(suite_glespy)



#!/usr/bin/env python2.7
# -*- coding: utf-8 -*-


__author__ = 'yarnaid'


class Printable(object):
    def __repr__(self):
        vals = []
        for k, v in self.__dict__.items():
            if k[0] != '_' and v not in ['', None]:
                vals.append('{}={}'.format(k, v))
        res = '<{}: {}>'.format(self.__class__.__name__, ', '.join(vals))
        return res

class Multipoled(Printable):

    """
    Class that has multipoles propreties
    """
    lmax = 1
    lmin = 0

    def __setattr__(self, name, value):
        if (name in ['lmax', 'lmin']) and (not isinstance(value, int)):
            raise ValueError('Multipoles must be an integer')
        self.__dict__[name] = max(0, value)
        self.__update_lmax()

    def __update_lmax(self):
        if self.lmin > self.lmax:
            self.__dict__['lmax'] = self.lmin


class Rendered(Multipoled):

    """
    Class with nx and np
    np >= nx * 2 >= lmax * 2 + 1 >= lmin
    """
    nx = 0
    np = 0

    def __init__(self, **kwargs):
        super(Rendered, self).__init__()
        for k, v in kwargs.items():
            self.__setattr__(k, v)

    def __setattr__(self, name, value):
        if (name in ['nx', 'np']) and (not isinstance(value, int)):
            raise ValueError('Map size must be an integer')
        self.__dict__[name] = value
        self.__update_data()
        self.__update_data_reverse()

    def __update_data(self):
        if self.lmin > self.lmax:
            self.__dict__['lmax'] = self.lmin
        if self.lmax * 2 + 1 > self.nx:
            self.__dict__['nx'] = self.lmax * 2 + 1
        if self.nx * 2 > self.np:
            self.__dict__['np'] = self.nx * 2

    def __update_data_reverse(self):
        if self.nx < 4:
            self.__dict__['nx'] = int(self.np / 2)
        if self.lmax < 2:
            self.__dict__['lmax'] = int((self.nx - 1) / 2)



#!/usr/bin/env python2.7
# -*- coding: utf-8 -*-

__author__ = 'yarnaid'

import unittest
import os

import glespy.test_data.data as test_data
import glespy.tools.convertion as convert
import glespy.ext.angles as angles


class FunctionsTests(unittest.TestCase, test_data.WithTestData):

    def setUp(self):
        self.init_data()
        print(self.alm_100_name)
        self.check_existance(self.alm_100_name)
        self.check_existance(self.map_name)

    def check_existance(self, file_name):
        self.assertTrue(os.path.exists(file_name))
        self.assertGreater(os.path.getsize(file_name), 0)

    def test_get_map_attrs(self):
        """
        check getting attributes of map
        """
        attrs = convert.get_map_attrs(map_name=self.map_name)
        self.assertEqual(self.attrs, attrs)

    def test_map_to_alm_good(self):
        """
        check of map convert to alm
        """
        alm_from_map = os.path.join(self.data_path, 'alm_test.fit')
        alm = convert.map_to_alm(
            map_name=self.map_name, alm_name=alm_from_map)
        self.check_existance(alm_from_map)
        os.remove(alm_from_map)

    def test_map_to_alm_no_alm(self):
        """
        check of map convert to alm
        """
        alm_name = convert.map_to_alm(map_name=self.map_name)
        self.check_existance(alm_name)
        os.remove(alm_name)

    # @unittest.skip('too long')
    def test_map_to_gif_no_gif(self):
        gif_name = convert.map_to_gif(self.map_name)
        self.check_existance(gif_name)
        os.remove(gif_name)

    @unittest.skip('Not important test')
    def test_show_map(self):
        convert.show_map(self.map_name)

    def test_kwargs_to_glesp_2(self):
        d = {'lmax': '10', 'n': 'str'}
        l = ['-lmax', '10', '-n', 'str'].sort()
        self.assertEqual(convert.kwargs_to_glesp_args(**d).sort(), l)

    def test_alm_to_map_with_lmax(self):
        mp = convert.alm_to_map(self.alm_100_name, lmax=100)
        self.check_existance(mp)
        os.remove(mp)

    def test_alm_to_map_with_l(self):
        mp = convert.alm_to_map(self.alm_100_name, l=33)
        self.check_existance(mp)
        os.remove(mp)

    def test_alm_to_map_with_nx(self):
        mp = convert.alm_to_map(self.alm_100_name, nx=123)
        self.check_existance(mp)
        os.remove(mp)

    def test_alm_to_map_with_np(self):
        mp = convert.alm_to_map(self.alm_100_name, np=231)
        self.check_existance(mp)
        os.remove(mp)

    def test_map_to_hist_with_name_and_hn(self):
        hist = convert.map_to_hist(self.map_name, hn=20)
        self.assertEqual(len(hist), 20)
        self.assertEqual(len(hist[-1]), 2)

    # @unittest.skip('very long...')
    def test_mask_map_without_name(self):
        masked = convert.mask_map(self.map_name, mask_name=self.mask_name)
        self.check_existance(masked)
        os.remove(masked)

    def test_points_to_map_without_name_with_error(self):
        self.assertRaises(
            ValueError, convert.points_to_map,
            points_name=self.points_name)

    def test_points_to_map_without_name(self):
        points_map = convert.points_to_map(self.points_name, nx=201)
        self.check_existance(points_map)
        os.remove(points_map)

    def test_smooth_alm_without_name(self):
        smoothed = convert.smooth_alm(self.alm_100_name, 300)
        self.check_existance(smoothed)
        os.remove(smoothed)

    def test_correlate(self):
        correlated_map = convert.correlate(self.map_name,
                                           convert.points_to_map(
                                           self.points_name,
                                           **self.attrs),
                                           180)
        self.check_existance(correlated_map)
        os.remove(correlated_map)

    def test_sum_maps(self):
        m1 = convert.points_to_map(self.points_name, **self.attrs)
        m2 = self.map_name
        sum = convert.sum_map(m1, m2)
        self.check_exist(sum)

    def test_rotate_alm(self):
        rotated = convert.rotate_alm(
            alm_name=self.alm_100_name, dphi=2, dtheta=2)
        self.check_existance(rotated)
        os.remove(rotated)

    def test_cut_map_circle(self):
        cutted = convert.cut_map_zone(self.map_name, angles.Zone(90, 0))
        self.check_existance(cutted)

    def test_alm_rotate_with_angle(self):
        rotated = convert.rotate_alm(
            self.alm_100_name,
            angle=angles.Angle(10, 100)
        )
        self.check_existance(rotated)

    def test_reflaction(self):
        r_map = convert.reflect_map(
            map_name=self.map_name,
            rtype='h'
        )
        self.check_existance(r_map)

    def test_mult(self):
        mult = convert.mult_map(
            self.map_name,
            mult=-1000,
        )
        self.check_existance(mult)

    def test_cl_to_map(self):
        rand_map = convert.cl_to_map(self.cl_name, lmax=100)['name']
        self.check_existance(rand_map)

    def test_cl_to_alm(self):
        rand_alm = convert.cl_to_alm(self.cl_name, lmax=100)['name']
        self.check_existance(rand_alm)



#!/usr/bin/env python
__author__ = 'yarnaid'

import unittest
try:
    import properties
except:
    import glespy.properties as properties


class MultipoledTests(unittest.TestCase):

    m = properties.Multipoled()

    def test_set_lmax_g_lmin(self):
        """
        if lmax > lmin
        """
        self.m.lmax = 2
        self.m.lmin = self.m.lmax - 1
        self.assertGreater(self.m.lmax, self.m.lmin)
        print('test!')

    def test_set_lmax_e_lmin(self):
        """
        lmax == lmin
        """
        self.m.lmax = 2
        self.m.lmin = self.m.lmax
        self.assertEqual(self.m.lmax, self.m.lmin)

    def test_set_lmin_g_lmax(self):
        """
        lmin > lmax
        """
        self.m.lmin = 10
        self.m.lmax = 1
        self.assertGreaterEqual(self.m.lmax, self.m.lmin)

    def test_set_lmax_g_lmin(self):
        """
        lmax < lmin
        """
        self.m.lmax = 1
        self.m.lmin = 10
        self.assertGreaterEqual(self.m.lmax, self.m.lmin)


class RenderedTests(unittest.TestCase):

    data = properties.Rendered()

    def check_lmax_nx(self):
        self.assertGreaterEqual(self.data.nx, self.data.lmax * 2 + 1)

    def check_nx_np(self):
        self.assertGreaterEqual(self.data.np, self.data.nx * 2)

    def setUp(self):
        self.data.lmin = 1
        self.data.lmax = 10

    def test_np_l_nx(self):
        """
         np < nx
        """
        self.data.np = 3
        self.data.nx = 10
        self.check_nx_np()

    def test_np_e_nx(self):
        """
        np = nx
        """
        self.data.np = 5
        self.data.nx = 5
        self.check_nx_np()

    def test_np_g_nx(self):
        """
        np > nx
        """
        self.data.np = 234
        self.data.nx = 3
        self.check_nx_np()

    def test_nx_l_np(self):
        """
        nx < np
        """
        self.data.nx = 12
        self.data.np = 523
        self.check_nx_np()

    def test_nx_g_np(self):
        """
        nx > np
        """
        self.data.nx = 300
        self.data.np = 3
        self.check_nx_np()

    def test_lmax_l_nx(self):
        """
        lmax < nx
        """
        self.data.lmax = 1
        self.data.nx = 100
        self.check_lmax_nx()

    def test_lmax_e_nx(self):
        """
        lmax == nx
        """
        self.data.lmax = 100
        self.data.nx = 100
        self.check_lmax_nx()

    def test_lmax_g_nx(self):
        """
        lmax > nx
        """
        self.data.lmax = 100
        self.data.nx = 1
        self.check_lmax_nx()

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python2.7
# -*- coding: utf-8 -*-
__author__ = 'yarnaid'

import logging
try:
    import colorer
except:
    import glespy.tools.colorer


logging.basicConfig(
    format='[%(funcName)s][%(asctime)s] %(levelname)s: %(message)s')
logging.basicConfig(datefmt='%m/%d/%Y %I:%M:%S %p')
logging.basicConfig(level=logging.INFO)
logging.basicConfig(levelname=logging.INFO)


logger = logging.getLogger()
logger.INFO = logging.INFO
logger.DEBUG = logging.DEBUG
logger.CRITICAL = logging.CRITICAL
logger.WARNING = logging.WARNING
logger.ERROR = logging.ERROR
logger.NOTSET = logging.NOTSET
logger.setLevel(logger.INFO)
logger.setLevel(logger.DEBUG)
# logger.setLevel(logger.CRITICAL)



#!/usr/bin/env python2.7
# -*- coding: utf-8 -*-

__author__ = 'yarnaid'


class Zone(object):

    """
    Zones for using with GlesPy
    LAT1,LON1,LAT2,LON2 or LAT,LON[,RADIUS]
    in degrees
    """
    angles = None

    def __init__(self, *args):
        super(Zone, self).__init__()
        if 2 > len(args) or len(args) > 4:
            raise IndexError(
                '*args number must be int [2, 3, 4] for circle or rectangle, {} got'.format(len(args)))
        if len(args) == 2:
            args += (90,)
        self.angles = args

    def __repr__(self):
        return 'd,'.join(map(str, self.angles)) + 'd'

    def __call__(self):
        return self.__repr__()

    def __invert__(self):
        """
        circle moves to dipole side
        rectangle save LAT1,LON1 and size changes direction
        """
        if len(self.angles) == 3:
            self.angles[0] = -self.angles[0]
            self.angles[1] += 180
            self.angles[2] = 180 - self.angles[2]
        if len(self.angles) == 4:
            self.angles[2] = self.angles[0] -\
                abs(self.angles[2] - self.angles[0])
            self.angles[3] = self.angles[1] -\
                abs(self.angles[3] - self.angles[1])


class Angle(object):

    """
    An angle of theta and phi
    """
    theta = None
    phi = None
    __deg = '{}d'

    def __init__(self, *args):
        super(Angle, self).__init__()
        if len(args) != 2:
            raise IndexError(
                '*args number must be 2, {} got'.format(len(args)),
            )
        self.theta = args[0]
        self.phi = args[1]

    def __invert__(self):
        """
        get an angle to opposite dipole side
        """
        return Angle(-self.theta, 180 + self.phi)

    def __add__(self, other):
        return Angle(self.theta + other.theta, self.phi + other.phi)

    def __radd__(self, other):
        return self.__add__(other)

    def __sub__(self, other):
        return Angle(self.theta - other.theta, self.phi - other.phi)

    # def __rsub__(self, other):
        # return self.__add__(-other)

    def __neg__(self):
        return Angle(-self.theta, -self.phi)

    def __pos__(self):
        return self

    def phi_d(self):
        return self.__deg.format(self.phi)

    def theta_d(self):
        return self.__deg.format(self.theta)



#!/usr/bin/env python2.7
# -*- coding: utf-8 -*-


__author__ = 'yarnaid'

import os
import cStringIO as StringIO
import subprocess as sp

import numpy as np

import glespy.tools.tools as tools
import glespy.properties as properties
from glespy.ext.angles import Zone, Angle


rtypes = ('c', 'h', 'v', 'p', 'C', 'H', 'V', 'P')


def check_file(name):
    if not os.path.exists(name):
        raise IOError('Fail in proceeding file {}'.format(name))
    return name


def get_map_attrs(map_name):
    check_file(map_name)
    output = tools.run_cmd(
        [tools.glesp['difmap'], '-st', map_name], debug_msg='getting attrs')
    output = output[0].split()
    output = list(map(lambda x: x.decode('utf-8'), output))
    try:
        res = {
            'nx': int(output[output.index('nx') + 2][0:-1]),
            'np': int(output[output.index('np') + 2]),
        }
    except:
        raise ValueError('Not a glesp file: "%s"' % (map_name))
    return res


def map_to_alm(map_name, alm_name=None, **kwargs):
    check_file(map_name)
    attrs = get_map_attrs(map_name)
    tmp_tresh_map = tools.get_out_name(suffix='map_to_alm_trash.fit')
    lmax = (attrs['nx'] - 1) / 2
    alm_name = tools.get_out_name(alm_name, suffix='alm_from_map.fit')
    args = [tools.binaries['cl2map'], '-map', map_name,
            '-lmax', lmax, '-ao', alm_name, '-o', tmp_tresh_map]
    args.extend(kwargs_to_glesp_args(**kwargs))
    tools.run_cmd(args, debug_msg='map to alm')
    try:
        os.remove(tmp_tresh_map)
    except:
        pass
    check_file(alm_name)
    return alm_name


def map_to_gif(map_name, gif_name=None, **kwargs):
    check_file(map_name)
    gif_name = tools.get_out_name(gif_name, suffix='map.gif')
    args = [tools.binaries['f2fig'], map_name, '-o', gif_name]
    args.extend(kwargs_to_glesp_args(**kwargs))
    tools.run_cmd(args, debug_msg='map to gif')
    check_file(gif_name)
    return gif_name


def show_map(map_name, **kwargs):
    check_file(map_name)
    gif = map_to_gif(map_name, **kwargs)
    args = [tools.binaries['viewer'], gif]
    # args.extend(kwargs_to_glesp_args(**kwargs))
    tools.run_cmd(args, debug_msg='show map')


def kwargs_to_glesp_args(**kwargs):
    """
    convert dict of kwargs to format '-name' 'value'
    if value is None --- to '-name' only
    """
    res = []
    for k, v in kwargs.items():
        if v is not None:
            ext = ['-' + k, v]
        else:
            ext = ['-' + k]
        res.extend(ext)
    return res


def props_to_kwargs(prop):
    res = {}
    res.update(prop.__dict__)
    return res


def props_to_glesp_args(prop):
    """

    :param prop: GlesPy.properties.Rendered
    :return: list of args in format '-name' 'value'
    """
    kwargs = props_to_kwargs(prop)
    return kwargs_to_glesp_args(**kwargs)


def kwargs_props_to_glesp_args(**kwargs):
    """
    convert dict from kwargs to format '-name' 'value'
    WITH ADDING ALL PARAMS
    for using as arguments for glesp commands
    :param kwargs:
    :return:
    """
    prop = properties.Rendered(**kwargs)
    if 'l' in kwargs:  # todo: make it works!
        prop.lmin = kwargs['l']
        prop.lmax = kwargs['l']
        kwargs.pop('l')
    opt_args = props_to_glesp_args(prop)
    return opt_args


def alm_to_map(alm_name, map_name=None, **kwargs):
    check_file(alm_name)
    opt_args = kwargs_props_to_glesp_args(**kwargs)
    map_name = tools.get_out_name(map_name, suffix='alm_from_map.fit')
    args = [tools.binaries['cl2map'], '-falm', alm_name, '-o', map_name]
    args.extend(opt_args)
    tools.run_cmd(args, debug_msg='alm to map')
    check_file(map_name)
    return map_name


def map_to_hist(map_name, hn=20, **kwargs):
    check_file(map_name)
    tmp_hist = tools.get_out_name(suffix='hist_from_map.txt')
    args = [tools.binaries['difmap'], map_name,
            '-st', '-hn', hn, '-hf', tmp_hist]
    args.extend(kwargs_to_glesp_args(**kwargs))
    args = remove_nx_np(args)
    tools.run_cmd(args, debug_msg='map to hist')
    hist_data = np.loadtxt(tmp_hist)
    os.remove(tmp_hist)
    return hist_data


def remove_nx_np(bad_args):
    args = bad_args
    for k in ('-nx', '-np'):
        try:
            i = args.index(k)
            del args[i + 1], args[i]
        except:
            pass
    return args


def alm_to_spec(alm_name, cl=True, **kwargs):
    if len(set(['lmax', 'l', 'lmin']).intersection(kwargs)) < 1:
        raise ValueError(
            '[%s]: lmax or l or lmin must be given' % (alm_to_cl.__name__))
    opt_args = kwargs_props_to_glesp_args(**kwargs)
    args = [tools.binaries['alm2dl'], '-cl' if cl else '', alm_name]
    args.extend(opt_args)
    args = remove_nx_np(args)
    with open(os.devnull) as stderr:
        args = map(str, args)
        raw_data = sp.check_output(args, stderr=stderr)
    cl_data = np.loadtxt(StringIO.StringIO(raw_data), dtype=[('x', 'f32'), ('y', 'f32')])
    return cl_data


def alm_to_cl(alm_name, **kwargs):
    return alm_to_spec(alm_name=alm_name, cl=True, **kwargs)


def alm_to_dl(alm_name, **kwargs):
    return alm_to_spec(alm_name=alm_name, cl=False, **kwargs)


def mask_map(map_name, mask_name, masked_map_name=None, **kwargs):
    check_file(map_name)
    check_file(mask_name)
    masked_map_name = tools.get_out_name(
        masked_map_name, suffix='masked_from_map.fit')
    opt_args = kwargs_props_to_glesp_args(**kwargs)
    args = [tools.binaries['difmap'], map_name,
            '-mf', mask_name, '-o', masked_map_name]
    args.extend(opt_args)
    args = remove_nx_np(args)
    tools.run_cmd(args, debug_msg='mask map')
    check_file(masked_map_name)
    return masked_map_name


def asci_to_map(asci_name, tp, map_name=None, **kwargs):
    if tp not in ['fm', 'fp']:
        raise ValueError('asci type must be fm or fp, {} got'.format(tp))
    if len(set(['nx', 'np', 'lmax', 'l', 'lmin']).intersection(kwargs)) < 1:
        raise ValueError(
            '[{}]: nx or np or lmax or l or lmin must be given'.format(
                alm_to_cl.__name__)
        )
    map_name = tools.get_out_name(map_name, suffix='map_from_points.fit')
    opt_args = kwargs_props_to_glesp_args(**kwargs)
    args = [tools.binaries['mappat'], '-' + tp, asci_name, '-o', map_name]
    args.extend(opt_args)
    tools.run_cmd(args, debug_msg='point to map')
    return check_file(map_name)


def points_to_map(points_name, map_name=None, **kwargs):
    return asci_to_map(points_name, 'fp', map_name, **kwargs)


def ascii_map_to_map(asci_map_name, map_name=None, **kwargs):
    return asci_to_map(asci_map_name, 'fm', map_name, **kwargs)


def smooth_alm(alm_name, smooth_window, smoothed_name=None, **kwargs):
    check_file(alm_name)
    smoothed_name = tools.get_out_name(
        smoothed_name, suffix='smoothed_alm.fit')
    opt_args = []
    try:
        opt_args = kwargs_props_to_glesp_args(**kwargs)
    except:
        pass
    args = [tools.binaries['rsalm'], alm_name,
            '-fw', smooth_window, '-o', smoothed_name]
    args.extend(opt_args)
    tools.run_cmd(args, debug_msg='smooth alm')
    return check_file(smoothed_name)


def correlate(map1, map2, correlation_window, correlated_map=None, **kwargs):
    check_file(map1)
    check_file(map2)
    correlated_map = tools.get_out_name(
        correlated_map, suffix='correlated_map.fit')
    try:
        opt_args = kwargs_props_to_glesp_args(**kwargs)
    except:
        opt_args = []
    args = [tools.binaries['difmap'], map1, map2,
            '-cw', correlation_window, '-o', correlated_map]
    args.extend(opt_args)
    args = remove_nx_np(args)
    tools.run_cmd(args, debug_msg='correlate maps')
    return check_file(correlated_map)


def sum_map(map1_name, map2_name, mult=[1.0, -1.0], summed_map=None, **kwargs):
    check_file(map1_name)
    check_file(map2_name)
    attrs1 = get_map_attrs(map1_name)
    attrs2 = get_map_attrs(map2_name)
    if attrs1 != attrs2:
        raise ValueError(
            '[{}]: maps has different resolution'.format(__name__))
    summed_map = tools.get_out_name(summed_map, suffix='summed.fit')
    for i in range(1, 2, 1):
        k = 'c{}'.format(i)
        if k in kwargs.keys():
            mult[i] = kwargs[k]
            del kwargs[k]
    try:
        targs = kwargs.copy()
        targs.update(attrs1)
        opt_args = kwargs_props_to_glesp_args(**targs)
    except:
        opt_args = []
    args = [tools.binaries['difmap'], '-c1', mult[0],
            '-c2', mult[1], map1_name, map2_name, '-o', summed_map]
    args.extend(opt_args)
    args = remove_nx_np(args)
    tools.run_cmd(args, debug_msg='sum maps')
    return check_file(summed_map)


def rotate_alm(alm_name, rotated_name=None, angle=None, **kwargs):
    """
    phi and psi are angels in radians
    :param dphi
    :param dpsi
    :param dtheta
    :return: rotated_alm_name
    """
    if not isinstance(angle, (Angle, type(None))):
        raise TypeError(
            'Angle must be an instance of {} class, {}.got'.format(
                angle.Angle.__class__.__name__, type(angle),
            )
        )
    check_file(alm_name)
    rotated_name = tools.get_out_name(rotated_name, suffix='rotated_alm.fit')
    opt_args = kwargs.copy()
    if angle:
        for k in ['dphi', 'dtheta', 'dp', 'dt']:
            opt_args.pop(k, None)
        opt_args['dtheta'] = angle.theta_d()
        opt_args['dphi'] = angle.phi_d()

    opt_args = kwargs_to_glesp_args(**opt_args)
    args = [tools.binaries['difalm'], alm_name, '-o', rotated_name]
    args.extend(opt_args)
    tools.run_cmd(args, debug_msg='rotate alm')
    return check_file(rotated_name)


def cut_map_zone(map_name, zone, cutted_map=None, **kwargs):
    if not isinstance(zone, Zone):
        raise AssertionError('zone must be an instance of {}'.format(
            Zone.__class__.__name__
        ))
    check_file(map_name)
    cutted_map = tools.get_out_name(cutted_map, suffix='cutted_map.fit')
    opt_args = kwargs_to_glesp_args(**kwargs)
    args = [tools.binaries['difmap'], map_name, '-keep', zone(),
            '-o', cutted_map]
    args.extend(opt_args)
    args = remove_nx_np(args)
    tools.run_cmd(args, debug_msg='cut map zone')
    try:
        return check_file(cutted_map)
    except:
        tools.debug(args)
        raise IOError('Failed to calc file {}'.format(cutted_map))


def reflect_map(map_name, rtype, reflected_name=None, **kwargs):
    if rtype not in rtypes:
        raise ValueError(
            'Reflection type must be in {}, got {}'.format(rtypes, rtype)
        )
    reflected_name = tools.get_out_name(
        reflected_name, suffix='reflected_map.fit',
    )
    check_file(map_name)
    args = [tools.binaries['difmap'], map_name, '-o', reflected_name,
            '-mir{}'.format(rtype.upper()),
    ]
    opt_args = kwargs_to_glesp_args(**kwargs)
    args.extend(opt_args)
    args = remove_nx_np(args)
    tools.run_cmd(args, debug_msg='reflect map')
    return check_file(reflected_name)


def mult_map(map_name, mult, mult_name=None, **kwargs):
    check_file(map_name)
    mult_name = tools.get_out_name(
        mult_name,
        suffix='multed_{}_map_fit'.format(mult),
    )
    opt_args = kwargs_to_glesp_args(**kwargs)
    args = [tools.binaries['difmap'], map_name, '-cf', mult, '-o', mult_name]
    args.extend(opt_args)
    args = remove_nx_np(args)
    tools.run_cmd(args, debug_msg='multiplying map')
    return check_file(mult_name)


def alm_update_ls(alm_name, update_alm_name=None, **kwargs):
    check_file(alm_name)
    update_alm_name = tools.get_out_name(update_alm_name, 'update_alm.fit')
    args = [tools.binaries['difalm'], alm_name, '-o', update_alm_name]
    opt_args = kwargs_to_glesp_args(**kwargs)
    args.extend(opt_args)
    args = remove_nx_np(args)
    tools.run_cmd(args, debug_msg='update alm l')
    return check_file(update_alm_name)


def cl_to_smth(cl_name, out_key='o', lmax=None, nx=None, lmin=0, smth_name=None, **kwargs):
    if out_key not in ('o', 'ao'):
        raise ValueError('out_key must be in ("o", "ao") for pixelmap and alm, {} got'.format(out_key))
    check_file(cl_name)
    smth_name = tools.get_out_name(smth_name, suffix='rand_alm_from_cl')
    if not lmax:
        try:
            cl_data = np.loadtxt(cl_name, dtype=('f8', 'f8'))
        except:
            raise IOError('Cannot read cl file {}'.format(cl_name))
        lmax = int(cl_data.T[0].max())
        lmin = int(cl_data.T[0].min())
    if not nx:
        nx = lmax * 2 + 1
    opt_args = kwargs_to_glesp_args(**kwargs)
    args = [tools.binaries['cl2map'], '-Cl', cl_name,
            '-lmax', lmax, '-lmin', lmin, '-nx', nx, '-np', nx * 2,
            '-' + out_key, smth_name]
    if 'a' in out_key:
        args.append('-nomap')
    args.extend(opt_args)
    tools.run_cmd(args, debug_msg='cl to map')
    return {'name': check_file(smth_name),
            'lmax': lmax,
            'nx': nx,
            'lmin': lmin,
            'kwargs': kwargs,
    }


def cl_to_alm(cl_name, lmax=None, nx=None, lmin=0, alm_name=None, **kwargs):
    return cl_to_smth(cl_name,
                      out_key='ao',
                      lmax=lmax,
                      nx=nx,
                      lmin=lmin,
                      smth_name=alm_name,
                      **kwargs)


def cl_to_map(cl_name, lmax=None, nx=None, lmin=0, map_name=None, **kwargs):
    return cl_to_smth(cl_name,
                      out_key='o',
                      lmax=lmax,
                      nx=nx,
                      lmin=lmin,
                      smth_name=map_name,
                      **kwargs)



#!/usr/bin/env python2.7
# -*- coding: utf-8 -*-

__author__ = 'yarnaid'

import os

import properties as properties
import tools.convertion as conv
import alm
from ext import angles


class gPixelMap(properties.Rendered):
    """
    Class with lmax, lmin, nx, np attributes for map rendering
    """
    name = None
    temp = True

    def __init__(self, **kwargs):
        # super(gPixelMap, self).__init__(**kwargs)
        for k, v in kwargs.items():
            setattr(self, k, v)
        if self.name:
            if os.path.exists(self.name) and self.name.count('tmp') < 1:
                self.temp = False

    def unset_temp(self):
        self.temp = False

    def __setattr__(self, name, value):
        if name == 'name' and os.path.exists(value) and value.count('tmp') < 1:
            self.__dict__['temp'] = False
        try:
            return super(gPixelMap, self).__setattr__(name, value)
        except:
            self.__dict__[name] = value

    def from_alm(self):
        pass

    def __del__(self):
        if self.temp:
            os.remove(self.name)

    def show(self, **kwargs):
        # todo: test
        conv.show_map(self.name, **kwargs)

    def add_map(self, other, **kwargs):
        try:
            other_name = other.name
        except AttributeError:
            other_name = other
        except:
            raise ValueError(
                'Can not sum PixelMap with {}'.format(type(other)))
        self.name = conv.sum_map(self.name, other_name, **kwargs)

    def __add__(self, other):
        attrs = self.get_attrs()
        attrs['name'] = conv.sum_map(self.name, other.name)
        return gPixelMap(**attrs)

    def __invert__(self):
        inverted = conv.mult_map(
            map_name=self.name,
            mult=-1,
        )
        attrs = self.get_attrs()
        attrs['name'] = inverted
        return gPixelMap(**attrs)

    def __neg__(self):
        return self.__invert__()

    def __sub__(self, other):
        attrs = self.get_attrs()
        attrs['name'] = conv.sum_map(self.name, other.name, mult=[1, 1])
        return gPixelMap(**attrs)

    def mult_number(self, other, **kwargs):
        attrs = self.get_attrs()
        attrs['name'] = conv.mult_map(self.name, other, **kwargs)
        return gPixelMap(**attrs)

    def __mul__(self, other):
        res = None
        if isinstance(other, (float, int,)):
            res = self.mult_number(other)
        return res

    def __rmul__(self, other):
        return self.__mul__(other)

    def __div__(self, other):
        return self.__mul__(other ** (-1))

    def get_attrs(self, **kwargs):
        res = {'nx': self.nx,
               'np': self.np,
               'lmax': self.lmax,
               'lmin': self.lmin,
        }
        res.update(kwargs)
        return res

    def get_keep_zone(self, zone, cutted_map=None, **kwargs):
        attrs = self.get_attrs(**kwargs)
        cutted_map = conv.cut_map_zone(
            map_name=self.name,
            zone=zone,
            cutted_map=cutted_map,
            **attrs
        )
        attrs['name'] = cutted_map
        return gPixelMap(**attrs)

    def get_half(self, lat, lon, map_name=None, **kwargs):
        return self.get_keep_zone(
            angles.Zone(lat, lon),
            cutted_map=map_name,
            **kwargs
        )

    def get_top(self, top_name=None, **kwargs):
        return self.get_half(-90, 0, top_name, **kwargs)

    def get_bottom(self, bottom_name=None, **kwargs):
        return self.get_half(90, 0, bottom_name, **kwargs)

    def get_reflection(self, rtype, reflected_name=None, **kwargs):
        attrs = self.get_attrs(**kwargs)
        reflected_name = conv.reflect_map(map_name=self.name,
                                          rtype=rtype,
                                          reflected_name=reflected_name,
                                          **attrs
        )
        attrs['name'] = reflected_name
        return gPixelMap(**attrs)

    def to_alm(self, alm_name=None, **kwargs):
        attrs = self.get_attrs(**kwargs)
        alm_name = conv.map_to_alm(self.name, alm_name, **attrs)
        attrs['name'] = alm_name
        return alm.Alm(**attrs)

    def to_cl(self, cl_name=None, **kwargs):
        return self.to_alm(**kwargs).to_cl()

    def to_dl(self, cl_name=None, **kwargs):
        return self.to_alm(**kwargs).to_dl()

    def get_masked(self, mask_name, masked_map_name=None, **kwargs):
        attrs = self.get_attrs(**kwargs)
        masked_map_name = conv.mask_map(
            self.name,
            mask_name,
            masked_map_name,
            **attrs
        )
        attrs['name'] = masked_map_name
        masked_map = gPixelMap(**attrs)
        return masked_map

    def to_gif(self, gif_name=None, **kwargs):
        return conv.map_to_gif(self.name, gif_name=gif_name, **kwargs)



__author__ = 'yarnaid'

import unittest
try:
    import pixelmap
    import test_data.data as data
    import pointsource
except:
    import glespy.pixelmap as pixelmap
    import glespy.test_data.data as data
    import glespy.pointsource as pointsource


class PixelMapTests(unittest.TestCase, data.WithTestData):

    def setUp(self):
        super(PixelMapTests, self).setUp()
        self.init_data()

    def check_params_with_lmax(self, pm, lmax):
        self.assertEqual(pm.lmax, 100)
        self.assertGreaterEqual(pm.lmax, pm.lmin)
        self.assertGreaterEqual(pm.nx, pm.lmax * 2 + 1)
        self.assertGreaterEqual(pm.np, pm.nx * 2)

    def test_creation(self):
        pm = pixelmap.gPixelMap(lmax=100)
        self.check_params_with_lmax(pm, 100)

    @unittest.expectedFailure
    def test_from_alm(self):
        self.check_exist('alm')

    def test_sum_with_ps(self):
        pm = pixelmap.gPixelMap(name=self.map_name, **self.attrs)
        ps = pointsource.PointSource(
            name=self.points_name).to_pixelmap(**self.attrs)
        pm.add_map(ps)
        self.check_exist(pm.name)



__author__ = 'yarnaid'

import unittest
import os

import glespy.tools.tools as tools

NT_FLG = ('nt' == os.name)


@unittest.skipIf(NT_FLG, 'GlespBinariesTest now is not for nt')
class GlespBinariesTest(unittest.TestCase):

    """
    Checking existence of glesp binaries
    """

    def test_binaries(self):
        for prog in tools.binaries.values():
            self.assertTrue(os.path.exists(prog))


class FunctionsTests(unittest.TestCase):

    @unittest.skipIf(NT_FLG, 'GlespBinariesTest now is not for nt')
    def test_run_cmd_mkdir(self):
        args = ['mkdir', '-p', 'my_dir']
        tools.run_cmd(args=args)
        self.assertTrue(os.path.exists('my_dir'))
        os.rmdir('my_dir')

    def test_run_cmd_stdout(self):
        """
        test stdout param of run_cmd
        """
        pass  # todo: add test_run_cmd_stdout

    def test_run_cmd_stderr(self):
        """
        test stderr param of run_cmd
        """
        pass  # todo: add test_run_cmd_stderr

    def test_get_out_name_with_name(self):
        """
        test for given 'name' param
        """
        n = 'my_name.tmp'
        nn = tools.get_out_name(n, delete=True)
        self.assertEqual(n, nn)

    def test_get_out_name_without_name(self):
        """
        should get name for tmp file and the file
        """
        n = tools.get_out_name(delete=False)
        self.assertTrue(os.path.exists(n))
        os.remove(n)

    def test_get_out_name_without_name_with_suffix(self):
        """
        test suffix in tmp file name
        """
        n = tools.get_out_name(suffix='suffix', delete=True)
        suffix = os.path.split(n)[-1].split('_')
        self.assertTrue('suffix' in suffix)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
__author__ = 'yarnaid'
try:
    import glespy.pointsource as pointsource
    import glespy.test_data.data as data
    # import glespy.pixelmap as pixelmap
except:
    import pointsource
    import test_data.data as data
    # import pixelmap

import unittest


class PointSourceTest(unittest.TestCase, data.WithTestData):

    def setUp(self):
        super(PointSourceTest, self).setUp()
        self.init_data()

    def test_to_pixelmap_with_nx(self):
        psource = pointsource.PointSource(name=self.points_name)
        p_map = psource.to_pixelmap(nx=101)
        self.check_exist(p_map.name)



__author__ = 'yarnaid'

import os
import inspect

map_name = 'test_map.fit'
alm_name = 'alm.fit'
alm_100_name = 'alm_100.fit'
mask_name = 'mask.fit'
points_name = 'points.dat'
cl_name = 'ilc_cl.txt'


class WithTestData(object):
    data_path = os.path.dirname(
        os.path.abspath(inspect.getfile(inspect.currentframe())))
    attrs = {
        'nx': 61,
        'np': 122,
    }

    def __init__(self):
        super().__init__()
        self.init_data()

    def check_exist(self, name):
        self.assertTrue(os.path.exists(name))
        self.assertGreater(os.path.getsize(name), 0)

    def init_data(self):
        self.map_name = os.path.join(self.data_path, map_name)
        self.alm_name = os.path.join(self.data_path, alm_name)
        self.alm_100_name = os.path.join(self.data_path, alm_100_name)
        self.mask_name = os.path.join(self.data_path, mask_name)
        self.points_name = os.path.join(self.data_path, points_name)
        self.cl_name = os.path.join(self.data_path, cl_name)
        pass  # todo: init maps with certain parameters



#!/usr/bin/env python2.7
# -*- coding: utf-8 -*-


__author__ = 'yarnaid'

import properties as properties
import tools.convertion as conv
import alm
import pixelmap
import os


class Cl(properties.Printable):
    """C(\ell)"""
    name = None
    temp = True
    data = None

    def __init__(self, **kwargs):
        #super(Cl, self).__init__(**kwargs)
        for k, v in kwargs.items():
            setattr(self, k, v)
        if self.name:
            if os.path.exists(self.name) and self.name.count('tmp') < 1:
                self.temp = False

    def to_alm(self, alm_name=None, **kwargs):
        attrs = {}
        alm_name = conv.tools.get_out_name(alm_name, suffix='rand_alm_from_cl')
        alm_res = conv.cl_to_alm(self.name, alm_name=alm_name, **kwargs)
        attrs.update(alm_res)
        return alm.Alm(**attrs)

    def to_map(self, map_name=None, **kwargs):
        attrs = {}
        map_name = conv.tools.get_out_name(map_name, suffix='rand_map_from_cl')
        map_res = conv.cl_to_map(self.name, map_name=map_name, **kwargs)
        attrs.update(map_res)
        return pixelmap.gPixelMap(**attrs)



__author__ = 'yarnaid'

import os
import subprocess as sp
import tempfile
import time

import glespy.tools.logger as lg


glesp_exec = '/usr/local/bin'
glesp = dict()
glesp['mappat'] = os.path.join(glesp_exec, 'mappat')
glesp['cl2map'] = os.path.join(glesp_exec, 'cl2map')
glesp['difmap'] = os.path.join(glesp_exec, 'difmap')
glesp['alm2dl'] = os.path.join(glesp_exec, 'alm2dl')
glesp['f2fig'] = os.path.join(glesp_exec, 'f2fig')
glesp['rsalm'] = os.path.join(glesp_exec, 'rsalm')
glesp['difalm'] = os.path.join(glesp_exec, 'difalm')
GIF_VIEWER = 'feh'

binaries = dict()
binaries.update(glesp)
binaries['viewer'] = os.path.join('/usr/bin', 'eog')

info = lg.logger.info
warn = lg.logger.warn
error = lg.logger.error
critical = lg.logger.critical
log = lg.logger.log
exception = lg.logger.exception
debug = lg.logger.debug

__all__ = ["run_cmd", "get_out_name", "glesp", "binaries",
           "info", "warn", "debug", "error", "critical", "log", "exception",
           ]


def run_cmd(args, debug_msg=None,
            shell=None,
            stdout=sp.PIPE,
            stderr=sp.PIPE,
            **kwargs):
    args = map(str, args)
    proc = sp.Popen(list(args),
                    stdout=stdout,
                    stderr=stderr,
                    shell=shell,
                    )
    proc.wait()
    res = proc.communicate()
    # debug(args[0])
    if debug_msg:
        debug(debug_msg)
        # for stream in res:
        # l = stream.readlines()
        # if len(l) > 0:
        # debug(l)
    if len(res[0]):
        debug(args)
        debug(res[0])
    if len(res[1]):
        debug([res[1]])
    return res


def get_out_name(out_name=None, suffix='tmp', dir='/tmp', **kwargs):
    # dir not tested!!!
    res = out_name
    if not res:
        tmp_file = tempfile.NamedTemporaryFile(dir=dir,
                                               suffix='_'+'_'.join(
                                                   [str(time.time()), suffix]),
                                               delete=kwargs.get('delete', True))
        res = tmp_file.name
    return res



#!/usr/bin/env python2.7
# -*- coding: utf-8 -*-

from distutils.core import setup
# import os
try:
    import pandoc
    pandoc.core.PANDOC_PATH = '/usr/bin/pandoc'

    doc = pandoc.Document()
    doc.markdown = open('README.md').read()
    f = open('README.txt', 'w+')
    f.write(doc.rst)
    f.close()

    long_description = open('README.txt').read()
except:
    long_description = ''

setup(
    name='glespy',
    version='0.1.7.2',
    packages=['glespy', 'glespy.test', 'glespy.tools',
              'glespy.tools.test', 'glespy.test_data',
              'glespy.ext',],
    url='https://pypi.python.org/pypi/glespy/',
    # license='no license yet...',
    author='yarnaid',
    author_email='yarnaid@gmail.com',
    description='Bindings for GLESP for calculations with spherical harmonics',
    keywords='GLESP bindings, cmb, spherical fucntions',
    long_description=long_description,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Topic :: Utilities",
        "Intended Audience :: Science/Research",
        "License :: Freeware",
        "Natural Language :: Russian",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 2",
        "Topic :: Scientific/Engineering :: Astronomy",
    ],
)



#!/usr/bin/env python2.7
# -*- coding: utf-8 -*-


__author__ = 'yarnaid'

import tools.convertion as convert
import pixelmap as pixelmap
import properties as properties
import os


class Alm(properties.Multipoled):

    name = None
    temp = True

    def get_attrs(self, **kwargs):
        res = {'lmax': self.lmax,
               'lmin': self.lmin}
        res.update(kwargs)
        return res

    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)
        if os.path.exists(self.name) and self.name.count('tmp') < 1:
            self.temp = False

    def __setattr__(self, name, value):
        if name == 'name' and os.path.exists(value) and value.count('tmp') < 1:
            self.__dict__['temp'] = False
        try:
            return super(Alm, self).__setattr__(name, value)
        except:
            self.__dict__[name] = value
        return value

    def __del__(self):
        if self.temp:
            os.remove(self.name)

    def to_map(self, map_name=None, **kwargs):
        attrs = self.get_attrs(**kwargs)
        attrs['name'] = convert.alm_to_map(
            self.name,
            map_name=map_name,
            **attrs
        )
        pmap = pixelmap.gPixelMap(**attrs)
        return pmap

    def smooth(self, smooth_window, smoothed_name=None, **kwargs):
        smoothed_name = convert.smooth_alm(
            self.name,
            smooth_window,
            smoothed_name=smoothed_name,
            **kwargs
        )
        attrs = self.get_attrs(kwargs)
        attrs['name'] = smoothed_name
        alm_smoothed = Alm(**attrs)
        self.name = smoothed_name
        return alm_smoothed

    def get_rotation(self, rotated_name=None, **kwargs):
        attrs = self.get_attrs(**kwargs)
        rotated_name = convert.rotate_alm(
            self.name,
            rotated_name=rotated_name,
            **attrs
        )
        attrs['name'] = rotated_name
        rotated_alm = Alm(**attrs)
        return rotated_alm

    def rotate(self, **kwargs):
        rotated = self.get_rotation(**kwargs)
        self.__dict__.update(rotated.get_attrs(**kwargs))
        self.name = rotated.name

    def to_cl(self, **kwargs):
        return convert.alm_to_cl(self.name, **self.get_attrs(**kwargs))

    def to_dl(self, **kwargs):
        return convert.alm_to_dl(self.name, **self.get_attrs(**kwargs))

    def update_ls(self, update_name=None, **kwargs):
        self.name = convert.alm_update_ls(
            self.name, update_name, **self.get_attrs(**kwargs))
        return self.name



import os

__author__ = 'yarnaid'

import unittest
try:
    import test_data.data as Data
    import alm as Alm
except:
    import glespy.test_data.data as Data
    import glespy.alm as Alm


class AlmTests(unittest.TestCase, Data.WithTestData):

    def setUp(self):
        super(AlmTests, self).setUp()
        self.init_data()
        self.check_exist(self.alm_100_name)

    def check_exist(self, name):
        self.assertTrue(os.path.exists(name))
        self.assertGreater(os.path.getsize(name), 0)

    def test_to_map(self):
        alm = Alm.Alm(name=self.alm_100_name, lmax=100)
        pmap = alm.to_map(nx=30)
        self.check_exist(pmap.name)
        os.remove(pmap.name)



# -*- coding: utf-8 -*-
#import os
__author__ = 'yarnaid'

try:
    import glespy.properties as properties
    import glespy.pixelmap as pixelmap
    import glespy.tools.convertion as conv
except:
    import properties


class PointSource(object):

    """
    Class for storing file with point sources and
    converting it
    """
    name = None

    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)

    def to_pixelmap(self, map_name=None, tp='fp', **kwargs):
        map_name = conv.asci_to_map(self.name, tp, map_name=map_name, **kwargs)
        attrs = kwargs.copy()
        attrs['name'] = map_name
        pixmap = pixelmap.gPixelMap(**attrs)
        return pixmap

    def show(self, **kwargs):
        self.to_pixelmap(**kwargs).show(**kwargs)
